@@ -103,28 +103,26 @@ swfdec_as_date_days_in_year (int year)
 #define IS_LEAP(year) (swfdec_as_date_days_in_year ((year)) == 366)
 
 static double
-swfdec_as_date_days_since_utc_for_year (int year)
+swfdec_as_date_days_since_utc_for_year (double year)
 {
-  double year_big = year;
-
-  return (
-      365 * (year_big - 1970) +
-      floor (((year_big - 1969) / 4.0f)) -
-      floor (((year_big - 1901) / 100.0f)) +
-      floor (((year_big - 1601) / 400.0f))
+  return floor (
+      365 * (year - 1970) +
+      floor (((year - 1969) / 4.0f)) -
+      floor (((year - 1901) / 100.0f)) +
+      floor (((year - 1601) / 400.0f))
     );
 }
 
-static int
+static double
 swfdec_as_date_days_from_utc_to_year (double days)
 {
-  int low, high, pivot;
+  double low, high, pivot;
 
   low = floor ((days >= 0 ? days / 366.0 : days / 365.0)) + 1970;
   high = ceil ((days >= 0 ? days / 365.0 : days / 366.0)) + 1970;
 
   while (low < high) {
-    pivot = ((double)low + (double)high) / 2.0;
+    pivot = floor ((low + high) / 2.0);
 
     if (swfdec_as_date_days_since_utc_for_year (pivot) <= days) {
       if (swfdec_as_date_days_since_utc_for_year (pivot + 1) > days) {
@@ -145,7 +143,7 @@ swfdec_as_date_milliseconds_to_brokentime (double milliseconds,
     BrokenTime *brokentime)
 {
   double remaining;
-  int year;
+  double year;
 
   g_assert (brokentime != NULL);
 
@@ -472,10 +470,11 @@ swfdec_as_date_set_field (SwfdecAsContext *cx, SwfdecAsObject *object,
 	break;
       case FIELD_YEAR:
 	// NOTE: Test against double, not the integer
-	if (d >= 100 || d < 0)
-	  number -= 1900;
+	if (d >= 0 && d < 100)
+	  number += 1900;
 	// fall trough
       case FIELD_FULL_YEAR:
+	number -= 1900;
 	if (!isfinite (d)) {
 	  swfdec_as_date_set_brokentime_value (date, utc, field_offsets[field],
 	      cx, 0 - 1900);
@@ -555,7 +554,7 @@ swfdec_as_date_toString (SwfdecAsContext *cx, SwfdecAsObject *object,
   SWFDEC_AS_CHECK (SWFDEC_TYPE_AS_DATE, &date, "");
 
   if (!swfdec_as_date_is_valid (date)) {
-    SWFDEC_AS_VALUE_SET_STRING (ret, "Invalid Date");
+    SWFDEC_AS_VALUE_SET_STRING (ret, SWFDEC_AS_STR_Invalid_Date);
     return;
   }
 
@@ -759,12 +758,20 @@ swfdec_as_date_setTime (SwfdecAsContext *cx, SwfdecAsObject *object,
     guint argc, SwfdecAsValue *argv, SwfdecAsValue *ret)
 {
   SwfdecAsDate *date;
+  double d;
 
   SWFDEC_AS_CHECK (SWFDEC_TYPE_AS_DATE, &date, "");
 
-  if (argc > 0) {
-    swfdec_as_date_set_milliseconds_utc (date,
-	swfdec_as_value_to_integer (cx, &argv[0]));
+  if (argc > 0 &&
+      (cx->version > 6 || !SWFDEC_AS_VALUE_IS_UNDEFINED (&argv[0]))) {
+    d = swfdec_as_value_to_number (cx, &argv[0]);
+  } else {
+    d = NAN;
+  }
+  if (isfinite (d)) {
+    swfdec_as_date_set_milliseconds_utc (date, trunc (d));
+  } else {
+    swfdec_as_date_set_milliseconds_utc (date, NAN);
   }
 
   SWFDEC_AS_VALUE_SET_NUMBER (ret, date->milliseconds);
@@ -794,6 +801,9 @@ swfdec_as_date_setSeconds (SwfdecAsContext *cx, SwfdecAsObject *object,
     guint argc, SwfdecAsValue *argv, SwfdecAsValue *ret)
 {
   swfdec_as_date_set_field (cx, object, argc, argv, ret, FIELD_SECONDS, FALSE);
+
+  if (argc > 1)
+    swfdec_as_date_setMilliseconds (cx, object, argc - 1, argv + 1, ret);
 }
 
 SWFDEC_AS_NATIVE (103, 128 + 14, swfdec_as_date_setUTCSeconds)
@@ -802,6 +812,9 @@ swfdec_as_date_setUTCSeconds (SwfdecAsContext *cx, SwfdecAsObject *object,
     guint argc, SwfdecAsValue *argv, SwfdecAsValue *ret)
 {
   swfdec_as_date_set_field (cx, object, argc, argv, ret, FIELD_SECONDS, TRUE);
+
+  if (argc > 1)
+    swfdec_as_date_setUTCMilliseconds (cx, object, argc - 1, argv + 1, ret);
 }
 
 SWFDEC_AS_NATIVE (103, 13, swfdec_as_date_setMinutes)
@@ -810,6 +823,9 @@ swfdec_as_date_setMinutes (SwfdecAsContext *cx, SwfdecAsObject *object,
     guint argc, SwfdecAsValue *argv, SwfdecAsValue *ret)
 {
   swfdec_as_date_set_field (cx, object, argc, argv, ret, FIELD_MINUTES, FALSE);
+
+  if (argc > 1)
+    swfdec_as_date_setSeconds (cx, object, argc - 1, argv + 1, ret);
 }
 
 SWFDEC_AS_NATIVE (103, 128 + 13, swfdec_as_date_setUTCMinutes)
@@ -818,6 +834,9 @@ swfdec_as_date_setUTCMinutes (SwfdecAsContext *cx, SwfdecAsObject *object,
     guint argc, SwfdecAsValue *argv, SwfdecAsValue *ret)
 {
   swfdec_as_date_set_field (cx, object, argc, argv, ret, FIELD_MINUTES, TRUE);
+
+  if (argc > 1)
+    swfdec_as_date_setUTCSeconds (cx, object, argc - 1, argv + 1, ret);
 }
 
 SWFDEC_AS_NATIVE (103, 12, swfdec_as_date_setHours)
@@ -826,6 +845,9 @@ swfdec_as_date_setHours (SwfdecAsContext *cx, SwfdecAsObject *object,
     guint argc, SwfdecAsValue *argv, SwfdecAsValue *ret)
 {
   swfdec_as_date_set_field (cx, object, argc, argv, ret, FIELD_HOURS, FALSE);
+
+  if (argc > 1)
+    swfdec_as_date_setMinutes (cx, object, argc - 1, argv + 1, ret);
 }
 
 SWFDEC_AS_NATIVE (103, 128 + 12, swfdec_as_date_setUTCHours)
@@ -834,6 +856,9 @@ swfdec_as_date_setUTCHours (SwfdecAsContext *cx, SwfdecAsObject *object,
     guint argc, SwfdecAsValue *argv, SwfdecAsValue *ret)
 {
   swfdec_as_date_set_field (cx, object, argc, argv, ret, FIELD_HOURS, TRUE);
+
+  if (argc > 1)
+    swfdec_as_date_setUTCMinutes (cx, object, argc - 1, argv + 1, ret);
 }
 
 SWFDEC_AS_NATIVE (103, 11, swfdec_as_date_setDate)
@@ -860,6 +885,9 @@ swfdec_as_date_setMonth (SwfdecAsContext *cx, SwfdecAsObject *object,
     guint argc, SwfdecAsValue *argv, SwfdecAsValue *ret)
 {
   swfdec_as_date_set_field (cx, object, argc, argv, ret, FIELD_MONTHS, FALSE);
+
+  if (argc > 1)
+    swfdec_as_date_setDate (cx, object, argc - 1, argv + 1, ret);
 }
 
 SWFDEC_AS_NATIVE (103, 128 + 10, swfdec_as_date_setUTCMonth)
@@ -868,6 +896,9 @@ swfdec_as_date_setUTCMonth (SwfdecAsContext *cx, SwfdecAsObject *object,
     guint argc, SwfdecAsValue *argv, SwfdecAsValue *ret)
 {
   swfdec_as_date_set_field (cx, object, argc, argv, ret, FIELD_MONTHS, TRUE);
+
+  if (argc > 1)
+    swfdec_as_date_setUTCDate (cx, object, argc - 1, argv + 1, ret);
 }
 
 SWFDEC_AS_NATIVE (103, 20, swfdec_as_date_setYear)
@@ -876,6 +907,9 @@ swfdec_as_date_setYear (SwfdecAsContext *cx, SwfdecAsObject *object,
     guint argc, SwfdecAsValue *argv, SwfdecAsValue *ret)
 {
   swfdec_as_date_set_field (cx, object, argc, argv, ret, FIELD_YEAR, FALSE);
+
+  if (argc > 1)
+    swfdec_as_date_setMonth (cx, object, argc - 1, argv + 1, ret);
 }
 
 SWFDEC_AS_NATIVE (103, 9, swfdec_as_date_setFullYear)
@@ -885,6 +919,9 @@ swfdec_as_date_setFullYear (SwfdecAsContext *cx, SwfdecAsObject *object,
 {
   swfdec_as_date_set_field (cx, object, argc, argv, ret, FIELD_FULL_YEAR,
       FALSE);
+
+  if (argc > 1)
+    swfdec_as_date_setMonth (cx, object, argc - 1, argv + 1, ret);
 }
 
 SWFDEC_AS_NATIVE (103, 128 + 9, swfdec_as_date_setUTCFullYear)
@@ -894,6 +931,9 @@ swfdec_as_date_setUTCFullYear (SwfdecAsContext *cx, SwfdecAsObject *object,
 {
   swfdec_as_date_set_field (cx, object, argc, argv, ret, FIELD_FULL_YEAR,
       TRUE);
+
+  if (argc > 1)
+    swfdec_as_date_setUTCMonth (cx, object, argc - 1, argv + 1, ret);
 }
 
 // Static methods
@@ -1017,26 +1057,16 @@ swfdec_as_date_construct (SwfdecAsContext *cx, SwfdecAsObject *object,
   SwfdecAsDate *date;
 
   if (!cx->frame->construct) {
-    SwfdecAsValue val;
-    if (!swfdec_as_context_use_mem (cx, sizeof (SwfdecAsDate)))
-      return;
-    object = g_object_new (SWFDEC_TYPE_AS_DATE, NULL);
-    swfdec_as_object_add (object, cx, sizeof (SwfdecAsDate));
-    swfdec_as_object_get_variable (cx->global, SWFDEC_AS_STR_Date, &val);
-    if (SWFDEC_AS_VALUE_IS_OBJECT (&val)) {
-      swfdec_as_object_set_constructor (object,
-	  SWFDEC_AS_VALUE_GET_OBJECT (&val));
-    } else {
-      SWFDEC_INFO ("\"Date\" is not an object");
-    }
+    object = g_object_new (SWFDEC_TYPE_AS_DATE, "context", cx, NULL);
+    swfdec_as_object_set_constructor_by_name (object, SWFDEC_AS_STR_Date, NULL);
   }
 
   date = SWFDEC_AS_DATE (object);
 
   /* FIXME: find a general solution here */
-  if (SWFDEC_IS_PLAYER (SWFDEC_AS_OBJECT (date)->context)) {
+  if (SWFDEC_IS_PLAYER (swfdec_gc_object_get_context (date))) {
     date->utc_offset =
-      SWFDEC_PLAYER (SWFDEC_AS_OBJECT (date)->context)->priv->system->utc_offset;
+      SWFDEC_PLAYER (swfdec_gc_object_get_context (date))->priv->system->utc_offset;
   }
 
   // don't accept arguments when not constructing
@@ -1107,7 +1137,7 @@ swfdec_as_date_construct (SwfdecAsContext *cx, SwfdecAsObject *object,
 	    &num)) {
 	brokentime.day_of_month = num;
       } else {
-	date->milliseconds = d;
+	date->milliseconds += d;
       }
     } else {
       brokentime.day_of_month = 1;
@@ -1118,7 +1148,7 @@ swfdec_as_date_construct (SwfdecAsContext *cx, SwfdecAsObject *object,
 	    &num)) {
 	brokentime.hours = num;
       } else {
-	date->milliseconds = d;
+	date->milliseconds += d;
       }
     }
 
@@ -1127,7 +1157,7 @@ swfdec_as_date_construct (SwfdecAsContext *cx, SwfdecAsObject *object,
 	    &num)) {
 	brokentime.minutes = num;
       } else {
-	date->milliseconds = d;
+	date->milliseconds += d;
       }
     }
 
@@ -1136,7 +1166,7 @@ swfdec_as_date_construct (SwfdecAsContext *cx, SwfdecAsObject *object,
 	    &num)) {
 	brokentime.seconds = num;
       } else {
-	date->milliseconds = d;
+	date->milliseconds += d;
       }
     }
 
@@ -1151,7 +1181,7 @@ swfdec_as_date_construct (SwfdecAsContext *cx, SwfdecAsObject *object,
 	    &num)) {
 	brokentime.milliseconds += num;
       } else {
-	date->milliseconds = d;
+	date->milliseconds += d;
       }
     }
 
@@ -1169,18 +1199,11 @@ swfdec_as_date_new (SwfdecAsContext *context, double milliseconds,
     int utc_offset)
 {
   SwfdecAsObject *ret;
-  SwfdecAsValue val;
 
   g_return_val_if_fail (SWFDEC_IS_AS_CONTEXT (context), NULL);
 
-  if (!swfdec_as_context_use_mem (context, sizeof (SwfdecAsDate)))
-    return NULL;
-
-  ret = g_object_new (SWFDEC_TYPE_AS_DATE, NULL);
-  swfdec_as_object_add (ret, context, sizeof (SwfdecAsDate));
-  swfdec_as_object_get_variable (context->global, SWFDEC_AS_STR_Date, &val);
-  if (SWFDEC_AS_VALUE_IS_OBJECT (&val))
-    swfdec_as_object_set_constructor (ret, SWFDEC_AS_VALUE_GET_OBJECT (&val));
+  ret = g_object_new (SWFDEC_TYPE_AS_DATE, "context", context, NULL);
+  swfdec_as_object_set_constructor_by_name (ret, SWFDEC_AS_STR_Date, NULL);
 
   SWFDEC_AS_DATE (ret)->milliseconds = milliseconds;
   SWFDEC_AS_DATE (ret)->utc_offset = utc_offset;
