@@ -29,6 +29,7 @@
 #include "swfdec_as_strings.h"
 #include "swfdec_as_super.h"
 #include "swfdec_debug.h"
+#include "swfdec_movie.h"
 
 /**
  * SECTION:SwfdecAsFrame
@@ -78,7 +79,7 @@ swfdec_as_stack_iterator_init_arguments (SwfdecAsStackIterator *iter, SwfdecAsFr
   SwfdecAsContext *context;
 
   g_return_val_if_fail (iter != NULL, NULL);
-  g_return_val_if_fail (SWFDEC_IS_AS_FRAME (frame), NULL);
+  g_return_val_if_fail (frame != NULL, NULL);
 
   if (frame->argc == 0) {
     iter->i = iter->n = 0;
@@ -86,7 +87,7 @@ swfdec_as_stack_iterator_init_arguments (SwfdecAsStackIterator *iter, SwfdecAsFr
     iter->current = NULL;
     return NULL;
   }
-  context = SWFDEC_AS_OBJECT (frame)->context;
+  context = swfdec_gc_object_get_context (frame->target);
   if (frame->argv) {
     iter->stack = NULL;
     iter->current = (SwfdecAsValue *) frame->argv;
@@ -131,9 +132,9 @@ swfdec_as_stack_iterator_init (SwfdecAsStackIterator *iter, SwfdecAsFrame *frame
   SwfdecAsStack *stack;
 
   g_return_val_if_fail (iter != NULL, NULL);
-  g_return_val_if_fail (SWFDEC_IS_AS_FRAME (frame), NULL);
+  g_return_val_if_fail (frame != NULL, NULL);
 
-  context = SWFDEC_AS_OBJECT (frame)->context;
+  context = swfdec_gc_object_get_context (frame->target);
   iter->i = 0;
   stack = context->stack;
   if (context->frame == frame) {
@@ -225,7 +226,7 @@ swfdec_as_frame_push_block (SwfdecAsFrame *frame, const guint8 *start,
 {
   SwfdecAsFrameBlock block = { start, end, func, data };
 
-  g_return_if_fail (SWFDEC_IS_AS_FRAME (frame));
+  g_return_if_fail (frame != NULL);
   g_return_if_fail (start <= end);
   g_return_if_fail (start >= frame->block_start);
   g_return_if_fail (end <= frame->block_end);
@@ -237,13 +238,13 @@ swfdec_as_frame_push_block (SwfdecAsFrame *frame, const guint8 *start,
 }
 
 void
-swfdec_as_frame_pop_block (SwfdecAsFrame *frame)
+swfdec_as_frame_pop_block (SwfdecAsFrame *frame, SwfdecAsContext *cx)
 {
   SwfdecAsFrameBlockFunc func;
   gpointer data;
   SwfdecAsFrameBlock *block;
 
-  g_return_if_fail (SWFDEC_IS_AS_FRAME (frame));
+  g_return_if_fail (frame != NULL);
   g_return_if_fail (frame->blocks->len > 0);
 
   block = &g_array_index (frame->blocks, SwfdecAsFrameBlock, frame->blocks->len - 1);
@@ -259,21 +260,17 @@ swfdec_as_frame_pop_block (SwfdecAsFrame *frame)
     frame->block_end = frame->script->buffer->data + frame->script->buffer->length;
   }
   /* only call function after we popped the block, so the block can push a new one */
-  func (frame, data);
+  func (cx, frame, data);
 }
 
 /*** FRAME ***/
 
-G_DEFINE_TYPE (SwfdecAsFrame, swfdec_as_frame, SWFDEC_TYPE_AS_OBJECT)
-
 static void
-swfdec_as_frame_dispose (GObject *object)
+swfdec_as_frame_free (SwfdecAsFrame *frame)
 {
-  SwfdecAsFrame *frame = SWFDEC_AS_FRAME (object);
-
   /* pop blocks while state is intact */
   while (frame->blocks->len > 0)
-    swfdec_as_frame_pop_block (frame);
+    swfdec_as_frame_pop_block (frame, swfdec_gc_object_get_context (frame->target));
 
   /* clean up */
   g_slice_free1 (sizeof (SwfdecAsValue) * frame->n_registers, frame->registers);
@@ -287,121 +284,19 @@ swfdec_as_frame_dispose (GObject *object)
     swfdec_script_unref (frame->script);
     frame->script = NULL;
   }
-
-  G_OBJECT_CLASS (swfdec_as_frame_parent_class)->dispose (object);
 }
 
-static void
-swfdec_as_frame_mark (SwfdecAsObject *object)
+void
+swfdec_as_frame_init (SwfdecAsFrame *frame, SwfdecAsContext *context, SwfdecScript *script)
 {
-  SwfdecAsFrame *frame = SWFDEC_AS_FRAME (object);
-  guint i;
-
-  if (frame->next)
-    swfdec_as_object_mark (SWFDEC_AS_OBJECT (frame->next));
-  g_slist_foreach (frame->scope_chain, (GFunc) swfdec_as_object_mark, NULL);
-  if (frame->thisp)
-    swfdec_as_object_mark (frame->thisp);
-  if (frame->super)
-    swfdec_as_object_mark (frame->super);
-  swfdec_as_object_mark (frame->target);
-  swfdec_as_object_mark (frame->original_target);
-  if (frame->function)
-    swfdec_as_object_mark (SWFDEC_AS_OBJECT (frame->function));
-  for (i = 0; i < frame->n_registers; i++) {
-    swfdec_as_value_mark (&frame->registers[i]);
-  }
-  /* don't mark argv, it's const, others have to take care of it */
-  SWFDEC_AS_OBJECT_CLASS (swfdec_as_frame_parent_class)->mark (object);
-}
-
-static char *
-swfdec_as_frame_debug (SwfdecAsObject *object)
-{
-  SwfdecAsFrame *frame = SWFDEC_AS_FRAME (object);
-  GString *string = g_string_new ("");
-  SwfdecAsStackIterator iter;
-  SwfdecAsValue *val;
-  char *s;
-  guint i;
-
-  if (frame->thisp) {
-    s = swfdec_as_object_get_debug (frame->thisp);
-    g_string_append (string, s);
-    g_string_append (string, ".");
-    g_free (s);
-  }
-  g_string_append (string, frame->function_name);
-  g_string_append (string, " (");
-  i = 0;
-  for (val = swfdec_as_stack_iterator_init_arguments (&iter, frame); val && i < 4;
-      val = swfdec_as_stack_iterator_next (&iter)) {
-    if (i > 0)
-      g_string_append (string, ", ");
-    i++;
-    if (i == 3 && frame->argc > 4) {
-      g_string_append (string, "...");
-    } else {
-      s = swfdec_as_value_to_debug (val);
-      g_string_append (string, s);
-      g_free (s);
-    }
-  }
-  g_string_append (string, ")");
-  return g_string_free (string, FALSE);
-}
-
-static void
-swfdec_as_frame_class_init (SwfdecAsFrameClass *klass)
-{
-  GObjectClass *object_class = G_OBJECT_CLASS (klass);
-  SwfdecAsObjectClass *asobject_class = SWFDEC_AS_OBJECT_CLASS (klass);
-
-  object_class->dispose = swfdec_as_frame_dispose;
-
-  asobject_class->mark = swfdec_as_frame_mark;
-  asobject_class->debug = swfdec_as_frame_debug;
-}
-
-static void
-swfdec_as_frame_init (SwfdecAsFrame *frame)
-{
-  frame->function_name = "unnamed";
-  frame->blocks = g_array_new (FALSE, FALSE, sizeof (SwfdecAsFrameBlock));
-  frame->block_end = (gpointer) -1;
-}
-
-static void
-swfdec_as_frame_load (SwfdecAsFrame *frame)
-{
-  SwfdecAsContext *context = SWFDEC_AS_OBJECT (frame)->context;
-
-  frame->stack_begin = context->cur;
-  context->base = frame->stack_begin;
-  frame->next = context->frame;
-  context->frame = frame;
-  context->call_depth++;
-}
-
-SwfdecAsFrame *
-swfdec_as_frame_new (SwfdecAsContext *context, SwfdecScript *script)
-{
-  SwfdecAsFrame *frame;
-  gsize size;
-
-  g_return_val_if_fail (SWFDEC_IS_AS_CONTEXT (context), NULL);
-  g_return_val_if_fail (script != NULL, NULL);
+  g_return_if_fail (frame != NULL);
+  g_return_if_fail (SWFDEC_IS_AS_CONTEXT (context));
+  g_return_if_fail (script != NULL);
   
-  size = sizeof (SwfdecAsFrame) + sizeof (SwfdecAsValue) * script->n_registers;
-  if (!swfdec_as_context_use_mem (context, size))
-    return NULL;
-  frame = g_object_new (SWFDEC_TYPE_AS_FRAME, NULL);
-  swfdec_as_object_add (SWFDEC_AS_OBJECT (frame), context, size);
+  swfdec_as_frame_init_native (frame, context);
   frame->script = swfdec_script_ref (script);
-  frame->function_name = script->name;
-  SWFDEC_DEBUG ("new frame for function %s", frame->function_name);
+  SWFDEC_DEBUG ("new frame for function %s", script->name);
   frame->pc = script->main;
-  frame->scope_chain = g_slist_prepend (frame->scope_chain, frame);
   frame->n_registers = script->n_registers;
   frame->registers = g_slice_alloc0 (sizeof (SwfdecAsValue) * frame->n_registers);
   if (script->constant_pool) {
@@ -411,26 +306,21 @@ swfdec_as_frame_new (SwfdecAsContext *context, SwfdecScript *script)
       SWFDEC_ERROR ("couldn't create constant pool");
     }
   }
-  swfdec_as_frame_load (frame);
-  return frame;
 }
 
-SwfdecAsFrame *
-swfdec_as_frame_new_native (SwfdecAsContext *context)
+void
+swfdec_as_frame_init_native (SwfdecAsFrame *frame, SwfdecAsContext *context)
 {
-  SwfdecAsFrame *frame;
-  gsize size;
-
-  g_return_val_if_fail (SWFDEC_IS_AS_CONTEXT (context), NULL);
+  g_return_if_fail (frame != NULL);
+  g_return_if_fail (SWFDEC_IS_AS_CONTEXT (context));
   
-  size = sizeof (SwfdecAsFrame);
-  if (!swfdec_as_context_use_mem (context, size))
-    return NULL;
-  frame = g_object_new (SWFDEC_TYPE_AS_FRAME, NULL);
-  SWFDEC_DEBUG ("new native frame");
-  swfdec_as_object_add (SWFDEC_AS_OBJECT (frame), context, size);
-  swfdec_as_frame_load (frame);
-  return frame;
+  frame->blocks = g_array_new (FALSE, FALSE, sizeof (SwfdecAsFrameBlock));
+  frame->block_end = (gpointer) -1;
+  frame->stack_begin = context->cur;
+  context->base = frame->stack_begin;
+  frame->next = context->frame;
+  context->frame = frame;
+  context->call_depth++;
 }
 
 /**
@@ -450,8 +340,8 @@ swfdec_as_frame_return (SwfdecAsFrame *frame, SwfdecAsValue *return_value)
   SwfdecAsValue retval;
   SwfdecAsFrame *next;
 
-  g_return_if_fail (SWFDEC_IS_AS_FRAME (frame));
-  context = SWFDEC_AS_OBJECT (frame)->context;
+  g_return_if_fail (frame != NULL);
+  context = swfdec_gc_object_get_context (frame->target ? (gpointer) frame->target : (gpointer) frame->function);
   g_return_if_fail (frame == context->frame);
 
   /* save return value in case it was on the stack somewhere */
@@ -509,6 +399,7 @@ swfdec_as_frame_return (SwfdecAsFrame *frame, SwfdecAsValue *return_value)
     swfdec_as_stack_ensure_free (context, 1);
     *swfdec_as_stack_push (context) = retval;
   }
+  swfdec_as_frame_free (frame);
 }
 
 /**
@@ -524,7 +415,7 @@ swfdec_as_frame_return (SwfdecAsFrame *frame, SwfdecAsValue *return_value)
 void
 swfdec_as_frame_set_this (SwfdecAsFrame *frame, SwfdecAsObject *thisp)
 {
-  g_return_if_fail (SWFDEC_IS_AS_FRAME (frame));
+  g_return_if_fail (frame != NULL);
   g_return_if_fail (frame->thisp == NULL);
   g_return_if_fail (SWFDEC_IS_AS_OBJECT (thisp));
 
@@ -557,24 +448,35 @@ swfdec_as_frame_get_variable_and_flags (SwfdecAsFrame *frame, const char *variab
     SwfdecAsValue *value, guint *flags, SwfdecAsObject **pobject)
 {
   GSList *walk;
+  SwfdecAsContext *cx;
 
-  g_return_val_if_fail (SWFDEC_IS_AS_FRAME (frame), NULL);
+  g_return_val_if_fail (frame != NULL, NULL);
   g_return_val_if_fail (variable != NULL, NULL);
 
+  cx = swfdec_gc_object_get_context (frame->target);
+
   for (walk = frame->scope_chain; walk; walk = walk->next) {
     if (swfdec_as_object_get_variable_and_flags (walk->data, variable, value, 
 	  flags, pobject))
       return walk->data;
   }
   /* we've walked the scope chain down. Now look in the special objects. */
-  /* 1) the target */
-  if (swfdec_as_object_get_variable_and_flags (frame->target, variable, value, 
-	flags, pobject))
-    return frame->target;
+  /* 1) the target (if removed, use original target) */
+  if (SWFDEC_IS_MOVIE (frame->target) &&
+      SWFDEC_MOVIE(frame->target)->state < SWFDEC_MOVIE_STATE_DESTROYED) {
+    if (swfdec_as_object_get_variable_and_flags (frame->target, variable,
+	  value, flags, pobject))
+      return frame->target;
+  } else {
+    if (swfdec_as_object_get_variable_and_flags (frame->original_target,
+	  variable, value, flags, pobject))
+      return frame->original_target;
+  }
   /* 2) the global object */
-  if (swfdec_as_object_get_variable_and_flags (
-	SWFDEC_AS_OBJECT (frame)->context->global, variable, value, flags, pobject))
-    return SWFDEC_AS_OBJECT (frame)->context->global;
+  /* FIXME: ignored on version 4, but should it never be created instead? */
+  if (cx->version > 4 && swfdec_as_object_get_variable_and_flags (cx->global,
+	variable, value, flags, pobject))
+    return cx->global;
 
   return NULL;
 }
@@ -586,7 +488,7 @@ swfdec_as_frame_set_variable_and_flags (SwfdecAsFrame *frame, const char *variab
   SwfdecAsObject *pobject, *set;
   GSList *walk;
 
-  g_return_if_fail (SWFDEC_IS_AS_FRAME (frame));
+  g_return_if_fail (frame != NULL);
   g_return_if_fail (variable != NULL);
 
   set = NULL;
@@ -600,8 +502,8 @@ swfdec_as_frame_set_variable_and_flags (SwfdecAsFrame *frame, const char *variab
     }
   }
   if (set == NULL) {
-    if (local && frame->is_local) {
-      set = SWFDEC_AS_OBJECT (frame);
+    if (local && frame->activation) {
+      set = frame->activation;
     } else {
       set = frame->target;
     }
@@ -621,7 +523,7 @@ swfdec_as_frame_delete_variable (SwfdecAsFrame *frame, const char *variable)
   GSList *walk;
   SwfdecAsDeleteReturn ret;
 
-  g_return_val_if_fail (SWFDEC_IS_AS_FRAME (frame), FALSE);
+  g_return_val_if_fail (frame != NULL, FALSE);
   g_return_val_if_fail (variable != NULL, FALSE);
 
   for (walk = frame->scope_chain; walk; walk = walk->next) {
@@ -635,7 +537,7 @@ swfdec_as_frame_delete_variable (SwfdecAsFrame *frame, const char *variable)
   if (ret)
     return ret;
   /* 2) the global object */
-  return swfdec_as_object_delete_variable (SWFDEC_AS_OBJECT (frame)->context->global, variable);
+  return swfdec_as_object_delete_variable (swfdec_gc_object_get_context (frame->target)->global, variable);
 }
 
 /**
@@ -650,7 +552,7 @@ swfdec_as_frame_delete_variable (SwfdecAsFrame *frame, const char *variable)
 void
 swfdec_as_frame_set_target (SwfdecAsFrame *frame, SwfdecAsObject *target)
 {
-  g_return_if_fail (SWFDEC_IS_AS_FRAME (frame));
+  g_return_if_fail (frame != NULL);
   g_return_if_fail (target == NULL || SWFDEC_IS_AS_OBJECT (target));
 
   if (target) {
@@ -671,22 +573,21 @@ swfdec_as_frame_preload (SwfdecAsFrame *frame)
   SwfdecAsContext *context;
   SwfdecAsStackIterator iter;
 
-  g_return_if_fail (SWFDEC_IS_AS_FRAME (frame));
-
+  g_return_if_fail (frame != NULL);
 
   /* setup */
-  object = SWFDEC_AS_OBJECT (frame);
-  context = object->context;
+  context = swfdec_gc_object_get_context (frame->target ? (gpointer) frame->target : (gpointer) frame->function);
   script = frame->script;
   if (frame->script == NULL)
     goto out;
+  frame->activation = swfdec_as_object_new_empty (context);
+  object = frame->activation;
+  frame->scope_chain = g_slist_prepend (frame->scope_chain, object);
 
   /* create arguments and super object if necessary */
   if ((script->flags & (SWFDEC_SCRIPT_PRELOAD_ARGS | SWFDEC_SCRIPT_SUPPRESS_ARGS)) != SWFDEC_SCRIPT_SUPPRESS_ARGS) {
     SwfdecAsFrame *next;
     args = swfdec_as_array_new (context);
-    if (!args)
-      goto out;
     for (cur = swfdec_as_stack_iterator_init_arguments (&iter, frame); cur != NULL;
 	cur = swfdec_as_stack_iterator_next (&iter)) {
       swfdec_as_array_push (SWFDEC_AS_ARRAY (args), cur);
@@ -702,14 +603,16 @@ swfdec_as_frame_preload (SwfdecAsFrame *frame)
     } else {
       SWFDEC_AS_VALUE_SET_NULL (&val);
     }
-    swfdec_as_object_set_variable (args, SWFDEC_AS_STR_caller, &val);
+    swfdec_as_object_set_variable_and_flags (args, SWFDEC_AS_STR_caller, &val,
+	SWFDEC_AS_VARIABLE_HIDDEN | SWFDEC_AS_VARIABLE_PERMANENT);
 
     if (frame->function != NULL) {
       SWFDEC_AS_VALUE_SET_OBJECT (&val, SWFDEC_AS_OBJECT (frame->function));
     } else {
       SWFDEC_AS_VALUE_SET_NULL (&val);
     }
-    swfdec_as_object_set_variable (args, SWFDEC_AS_STR_callee, &val);
+    swfdec_as_object_set_variable_and_flags (args, SWFDEC_AS_STR_callee, &val,
+	SWFDEC_AS_VARIABLE_HIDDEN | SWFDEC_AS_VARIABLE_PERMANENT);
   } else {
     /* silence gcc */
     args = NULL;
@@ -815,13 +718,13 @@ swfdec_as_frame_handle_exception (SwfdecAsFrame *frame)
 {
   SwfdecAsContext *cx;
 
-  g_return_if_fail (SWFDEC_IS_AS_FRAME (frame));
-  cx = SWFDEC_AS_OBJECT (frame)->context;
+  g_return_if_fail (frame != NULL);
+  cx = swfdec_gc_object_get_context (frame->target);
   g_return_if_fail (cx->exception);
 
   /* pop blocks in the hope that we are inside a Try block */
   while (cx->exception && frame->blocks->len) {
-    swfdec_as_frame_pop_block (frame);
+    swfdec_as_frame_pop_block (frame, cx);
   }
   /* no Try blocks caught it, exit frame */
   if (cx->exception) {
@@ -841,29 +744,11 @@ swfdec_as_frame_handle_exception (SwfdecAsFrame *frame)
 SwfdecAsFrame *
 swfdec_as_frame_get_next (SwfdecAsFrame *frame)
 {
-  g_return_val_if_fail (SWFDEC_IS_AS_FRAME (frame), NULL);
+  g_return_val_if_fail (frame != NULL, NULL);
 
   return frame->next;
 }
 
-/**
- * swfdec_as_frame_get_function_name:
- * @frame: a #SwfdecAsFrame
- *
- * Gets the name of the function that is currently executing. This function is
- * intended for debugging purposes.
- *
- * Returns: a string. Do not free.
- **/
-const char *
-swfdec_as_frame_get_function_name (SwfdecAsFrame *frame)
-{
-  g_return_val_if_fail (SWFDEC_IS_AS_FRAME (frame), NULL);
-
-  g_assert (frame->function_name);
-  return frame->function_name;
-}
-
 /**
  * swfdec_as_frame_get_script:
  * @frame: a #SwfdecAsFrame
@@ -876,7 +761,7 @@ swfdec_as_frame_get_function_name (SwfdecAsFrame *frame)
 SwfdecScript *
 swfdec_as_frame_get_script (SwfdecAsFrame *frame)
 {
-  g_return_val_if_fail (SWFDEC_IS_AS_FRAME (frame), NULL);
+  g_return_val_if_fail (frame != NULL, NULL);
 
   return frame->script;
 }
@@ -893,7 +778,7 @@ swfdec_as_frame_get_script (SwfdecAsFrame *frame)
 SwfdecAsObject *
 swfdec_as_frame_get_this (SwfdecAsFrame *frame)
 {
-  g_return_val_if_fail (SWFDEC_IS_AS_FRAME (frame), NULL);
+  g_return_val_if_fail (frame != NULL, NULL);
 
   return frame->thisp;
 }
