@@ -25,11 +25,9 @@
 
 G_BEGIN_DECLS
 
-typedef void (* SwfdecAsFrameBlockFunc) (SwfdecAsFrame *frame, gpointer data);
+typedef void (* SwfdecAsFrameBlockFunc) (SwfdecAsContext *cx, SwfdecAsFrame *frame, gpointer data);
 
 struct _SwfdecAsFrame {
-  SwfdecAsObject	object;
-
   SwfdecAsFrame *	next;		/* next frame (FIXME: keep a list in the context instead?) */
   SwfdecAsFunction *	function;	/* function we're executing or NULL if toplevel */
   SwfdecAsObject *	thisp;		/* this object in current frame or NULL if none */
@@ -38,8 +36,6 @@ struct _SwfdecAsFrame {
   SwfdecAsValue *	return_value;	/* pointer to where to store the return value */
   guint			argc;		/* number of arguments */
   const SwfdecAsValue *	argv;		/* arguments or %NULL if taken from stack */
-  /* debugging */
-  const char *		function_name;	/* name of function */
   /* script execution */
   SwfdecScript *	script;		/* script being executed */
   GSList *		scope_chain;  	/* the scope chain (with objects etc) */
@@ -48,7 +44,7 @@ struct _SwfdecAsFrame {
   GArray *		blocks;		/* blocks we have entered (like With) */
   SwfdecAsObject *	target;		/* target to use as last object in scope chain or for SetVariable */
   SwfdecAsObject *	original_target;/* original target (used when resetting target) */
-  gboolean		is_local;	/* TRUE if this frame takes local variables */
+  SwfdecAsObject *	activation;	/* activation object or NULL if the frame takes no local variables */
   SwfdecAsValue *	registers;	/* the registers */
   guint			n_registers;	/* number of allocated registers */
   SwfdecConstantPool *	constant_pool;	/* constant pool currently in use */
@@ -57,13 +53,11 @@ struct _SwfdecAsFrame {
   /* native function */
 };
 
-struct _SwfdecAsFrameClass {
-  SwfdecAsObjectClass	object_class;
-};
-
-SwfdecAsFrame *	swfdec_as_frame_new		(SwfdecAsContext *	context,
+void		swfdec_as_frame_init		(SwfdecAsFrame *	frame,
+						 SwfdecAsContext *	context,
 						 SwfdecScript *		script);
-SwfdecAsFrame *	swfdec_as_frame_new_native	(SwfdecAsContext *	context);
+void		swfdec_as_frame_init_native	(SwfdecAsFrame *	frame,
+						 SwfdecAsContext *	context);
 void		swfdec_as_frame_return		(SwfdecAsFrame *	frame,
 						 SwfdecAsValue *	return_value);
 
@@ -99,7 +93,8 @@ void		swfdec_as_frame_push_block	(SwfdecAsFrame *	frame,
 						 const guint8 *		end,
 						 SwfdecAsFrameBlockFunc	func,
 						 gpointer		data);
-void		swfdec_as_frame_pop_block	(SwfdecAsFrame *	frame);
+void		swfdec_as_frame_pop_block	(SwfdecAsFrame *	frame,
+						 SwfdecAsContext *	context);
 void		swfdec_as_frame_handle_exception(SwfdecAsFrame *	frame);
 
 
