@@ -39,14 +39,10 @@ G_BEGIN_DECLS
 
 void		swfdec_as_function_set_constructor	(SwfdecAsFunction *	fun);
 void		swfdec_as_function_init_context		(SwfdecAsContext *	context);
-SwfdecAsFrame *	swfdec_as_function_call_no_preload	(SwfdecAsFunction *	function, 
-							 SwfdecAsObject *	thisp,
-							 guint			n_args,
-							 const SwfdecAsValue *	args, 
-							 SwfdecAsValue *	return_value);
 
 /* swfdec_as_context.c */
 gboolean	swfdec_as_context_check_continue (SwfdecAsContext *	context);
+void		swfdec_as_context_run		(SwfdecAsContext *	context);
 void		swfdec_as_context_run_init_script (SwfdecAsContext *	context,
 						 const guint8 *		data,
 						 gsize			length,
@@ -74,6 +70,10 @@ void		swfdec_as_object_add_native_variable (SwfdecAsObject *	object,
 						 const char *		variable,
 						 SwfdecAsNative		get,
 						 SwfdecAsNative		set);
+void		swfdec_as_object_set_constructor_by_name 
+						(SwfdecAsObject *	object,
+						 const char *		name,
+						 ...) G_GNUC_NULL_TERMINATED;
 
 
 G_END_DECLS
