@@ -112,11 +112,16 @@ swfdec_as_string_lastIndexOf (SwfdecAsContext *cx, SwfdecAsObject *object,
   s = swfdec_as_value_to_string (cx, &argv[0]);
   if (argc == 2) {
     int offset = swfdec_as_value_to_integer (cx, &argv[1]);
+    const char *tmp;
     if (offset < 0) {
       SWFDEC_AS_VALUE_SET_INT (ret, -1);
       return;
     }
-    len = g_utf8_offset_to_pointer (string, offset + 1) - string;
+    tmp = string;
+    offset++;
+    while (*tmp && offset-- != 0)
+      tmp = g_utf8_next_char (tmp);
+    len = tmp - string;
   } else {
     len = G_MAXSIZE;
   }
