@@ -511,5 +511,25 @@ const char swfdec_as_strings[] =
   SWFDEC_AS_CONSTANT_STRING ("lr")
   SWFDEC_AS_CONSTANT_STRING ("rl")
   SWFDEC_AS_CONSTANT_STRING ("rr")
+  SWFDEC_AS_CONSTANT_STRING ("flash")
+  SWFDEC_AS_CONSTANT_STRING ("geom")
+  SWFDEC_AS_CONSTANT_STRING ("ColorTransform")
+  SWFDEC_AS_CONSTANT_STRING ("Transform")
+  SWFDEC_AS_CONSTANT_STRING ("bufferLength")
+  SWFDEC_AS_CONSTANT_STRING ("bufferTime")
+  SWFDEC_AS_CONSTANT_STRING ("audiocodec")
+  SWFDEC_AS_CONSTANT_STRING ("currentFps")
+  SWFDEC_AS_CONSTANT_STRING ("decodedFrames")
+  SWFDEC_AS_CONSTANT_STRING ("liveDelay")
+  SWFDEC_AS_CONSTANT_STRING ("videoCodec")
+  SWFDEC_AS_CONSTANT_STRING ("System")
+  SWFDEC_AS_CONSTANT_STRING ("__resolve")
+  SWFDEC_AS_CONSTANT_STRING ("Rectangle")
+  SWFDEC_AS_CONSTANT_STRING ("BitmapData")
+  SWFDEC_AS_CONSTANT_STRING ("tx")
+  SWFDEC_AS_CONSTANT_STRING ("ty")
+  SWFDEC_AS_CONSTANT_STRING ("Invalid Date")
+  SWFDEC_AS_CONSTANT_STRING ("auto")
+  SWFDEC_AS_CONSTANT_STRING ("Matrix")
   /* add more here */
 ;
