@@ -46,11 +46,7 @@ swfdec_net_stream_onstatus (SwfdecNetStream *stream, const char *code, const cha
   SwfdecAsObject *object;
 
   swfdec_sandbox_use (stream->sandbox);
-  object = swfdec_as_object_new (SWFDEC_AS_OBJECT (stream)->context);
-  if (!object) {
-    swfdec_sandbox_unuse (stream->sandbox);
-    return;
-  }
+  object = swfdec_as_object_new (swfdec_gc_object_get_context (stream));
   SWFDEC_INFO ("emitting onStatus for %s %s", level, code);
   SWFDEC_AS_VALUE_SET_STRING (&val, code);
   swfdec_as_object_set_variable (object, SWFDEC_AS_STR_code, &val);
@@ -58,7 +54,21 @@ swfdec_net_stream_onstatus (SwfdecNetStream *stream, const char *code, const cha
   swfdec_as_object_set_variable (object, SWFDEC_AS_STR_level, &val);
 
   SWFDEC_AS_VALUE_SET_OBJECT (&val, object);
-  swfdec_as_object_call (SWFDEC_AS_OBJECT (stream), SWFDEC_AS_STR_onStatus, 1, &val, NULL);
+  if (!swfdec_as_object_call (SWFDEC_AS_OBJECT (stream),
+        SWFDEC_AS_STR_onStatus, 1, &val, NULL)) {
+    // if it's an error message and the stream object didn't have onStatus
+    // handler, call System.onStatus
+    if (level == SWFDEC_AS_STR_error) {
+      SwfdecAsValue system;
+
+      swfdec_as_object_get_variable (swfdec_gc_object_get_context (stream)->global,
+          SWFDEC_AS_STR_System, &system);
+      if (SWFDEC_AS_VALUE_IS_OBJECT (&system)) {
+        swfdec_as_object_call (SWFDEC_AS_VALUE_GET_OBJECT (&system),
+              SWFDEC_AS_STR_onStatus, 1, &val, NULL);
+      }
+    }
+  }
   swfdec_sandbox_unuse (stream->sandbox);
 }
 
@@ -82,14 +92,43 @@ swfdec_net_stream_decode_video (SwfdecVideoDecoder *decoder, SwfdecBuffer *buffe
     tmp = swfdec_buffer_new_subbuffer (buffer, 1, buffer->length - 1);
     swfdec_video_decoder_decode (decoder, tmp);
     swfdec_buffer_unref (tmp);
-    if (hsub || wsub) {
-      if (hsub >= decoder->height || wsub >= decoder->width) {
-	swfdec_video_decoder_error (decoder, "can't reduce size by more than available");
-	return;
-      }
+    if (hsub >= decoder->height || wsub >= decoder->width) {
+      SWFDEC_ERROR ("can't reduce size by more than available");
+      decoder->width = 0;
+      decoder->height = 0;
+    } else {
       decoder->width -= wsub;
       decoder->height -= hsub;
     }
+  } else if (decoder->codec == SWFDEC_VIDEO_CODEC_H264) {
+    SwfdecBits bits;
+    guint type;
+    SwfdecBuffer *data;
+    swfdec_bits_init (&bits, buffer);
+    type = swfdec_bits_get_u8 (&bits);
+    /* composition_time_offset = */ swfdec_bits_get_bu24 (&bits);
+    switch (type) {
+      case 0:
+	data = swfdec_bits_get_buffer (&bits, -1);
+	if (data) {
+	  swfdec_video_decoder_set_codec_data (decoder, data);
+	  swfdec_buffer_unref (data);
+	}
+	break;
+      case 1:
+	data = swfdec_bits_get_buffer (&bits, -1);
+	if (data) {
+	  swfdec_video_decoder_decode (decoder, data);
+	} else {
+	  SWFDEC_ERROR ("no data in H264 buffer?");
+	}
+	break;
+      case 2:
+	break;
+      default:
+	SWFDEC_ERROR ("H264 data type %u not supported", type);
+	break;
+      }
   } else {
     swfdec_video_decoder_decode (decoder, buffer);
   }
@@ -126,14 +165,11 @@ swfdec_net_stream_video_goto (SwfdecNetStream *stream, guint timestamp)
   } else {
     guint next;
 
-    if (stream->decoder && swfdec_video_decoder_get_codec (stream->decoder) != format) {
-      g_object_unref (stream->decoder);
-      stream->decoder = NULL;
-    }
     if (stream->decoder != NULL &&
-	(stream->decoder_time >= stream->current_time)) {
-      g_object_unref (stream->decoder);
-      stream->decoder = NULL;
+	stream->decoder_time >= stream->current_time) {
+      buffer = swfdec_flv_decoder_get_video (stream->flvdecoder, 
+	  stream->current_time, TRUE, &format, &stream->decoder_time,
+	  &next);
     }
 
     if (stream->decoder == NULL) {
@@ -205,7 +241,7 @@ swfdec_net_stream_video_goto (SwfdecNetStream *stream, guint timestamp)
       process_events_from++; /* increase so we get the next event next time */
       swfdec_bits_init (&bits, event);
       swfdec_sandbox_use (stream->sandbox);
-      if (swfdec_amf_parse (SWFDEC_AS_OBJECT (stream)->context, &bits, 2, 
+      if (swfdec_amf_parse (swfdec_gc_object_get_context (stream), &bits, 2, 
 	    SWFDEC_AMF_STRING, &name, SWFDEC_AMF_MIXED_ARRAY, &value) != 2) {
 	SWFDEC_ERROR ("could not parse data tag");
       } else {
@@ -230,14 +266,14 @@ swfdec_net_stream_timeout (SwfdecTimeout *timeout)
       stream->timeout.callback) {
     SWFDEC_LOG ("readding timeout");
     stream->timeout.timestamp += SWFDEC_MSECS_TO_TICKS (stream->next_time - stream->current_time);
-    swfdec_player_add_timeout (SWFDEC_PLAYER (SWFDEC_AS_OBJECT (stream)->context), &stream->timeout);
+    swfdec_player_add_timeout (SWFDEC_PLAYER (swfdec_gc_object_get_context (stream)), &stream->timeout);
   }
 }
 
 static void
 swfdec_net_stream_update_playing (SwfdecNetStream *stream)
 {
-  SwfdecPlayer *player = SWFDEC_PLAYER (SWFDEC_AS_OBJECT (stream)->context);
+  SwfdecPlayer *player = SWFDEC_PLAYER (swfdec_gc_object_get_context (stream));
   gboolean should_play;
     
   should_play = stream->playing; /* checks user-set play/pause */
@@ -277,7 +313,7 @@ swfdec_net_stream_update_playing (SwfdecNetStream *stream)
 static SwfdecPlayer *
 swfdec_net_stream_stream_target_get_player (SwfdecStreamTarget *target)
 {
-  return SWFDEC_PLAYER (SWFDEC_AS_OBJECT (target)->context);
+  return SWFDEC_PLAYER (swfdec_gc_object_get_context (target));
 }
 
 static void
@@ -323,7 +359,7 @@ swfdec_net_stream_stream_target_parse (SwfdecStreamTarget *target,
     /* FIXME: add mp3 support */
     ns->flvdecoder = g_object_new (SWFDEC_TYPE_FLV_DECODER, NULL);
     g_signal_connect_swapped (ns->flvdecoder, "missing-plugin", 
-	G_CALLBACK (swfdec_player_add_missing_plugin), SWFDEC_AS_OBJECT (ns)->context);
+	G_CALLBACK (swfdec_player_add_missing_plugin), swfdec_gc_object_get_context (ns));
     swfdec_net_stream_onstatus (ns, SWFDEC_AS_STR_NetStream_Play_Start,
 	SWFDEC_AS_STR_status);
     swfdec_loader_set_data_type (SWFDEC_LOADER (stream), SWFDEC_LOADER_DATA_FLV);
@@ -506,33 +542,49 @@ swfdec_net_stream_get_variable (SwfdecAsObject *object, SwfdecAsObject *orig,
     SWFDEC_AS_VALUE_SET_NUMBER (val, bytes);
     *flags = 0;
     return TRUE;
+  } else if (variable == SWFDEC_AS_STR_bufferLength) {
+    SWFDEC_STUB ("Netstream.bufferLength (get)");
+  } else if (variable == SWFDEC_AS_STR_bufferTime) {
+    SWFDEC_STUB ("Netstream.bufferTime (get)");
+  } else if (variable == SWFDEC_AS_STR_audiocodec) {
+    SWFDEC_STUB ("Netstream.audiocodec (get)");
+  } else if (variable == SWFDEC_AS_STR_currentFps) {
+    SWFDEC_STUB ("Netstream.currentFps (get)");
+  } else if (variable == SWFDEC_AS_STR_decodedFrames) {
+    SWFDEC_STUB ("Netstream.decodedFrames (get)");
+  } else if (variable == SWFDEC_AS_STR_liveDelay) {
+    SWFDEC_STUB ("Netstream.liveDelay (get)");
+  } else if (variable == SWFDEC_AS_STR_videoCodec) {
+    SWFDEC_STUB ("Netstream.videoCodec (get)");
   }
   return FALSE;
 }
 
 static void
-swfdec_net_stream_mark (SwfdecAsObject *object)
+swfdec_net_stream_mark (SwfdecGcObject *object)
 {
   SwfdecNetStream *stream = SWFDEC_NET_STREAM (object);
 
   if (stream->conn)
-    swfdec_as_object_mark (SWFDEC_AS_OBJECT (stream->conn));
+    swfdec_gc_object_mark (stream->conn);
   if (stream->sandbox)
-    swfdec_as_object_mark (SWFDEC_AS_OBJECT (stream->sandbox));
+    swfdec_gc_object_mark (stream->sandbox);
 
-  SWFDEC_AS_OBJECT_CLASS (swfdec_net_stream_parent_class)->mark (object);
+  SWFDEC_GC_OBJECT_CLASS (swfdec_net_stream_parent_class)->mark (object);
 }
 
 static void
 swfdec_net_stream_class_init (SwfdecNetStreamClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
+  SwfdecGcObjectClass *gc_class = SWFDEC_GC_OBJECT_CLASS (klass);
   SwfdecAsObjectClass *asobject_class = SWFDEC_AS_OBJECT_CLASS (klass);
 
   object_class->dispose = swfdec_net_stream_dispose;
 
+  gc_class->mark = swfdec_net_stream_mark;
+
   asobject_class->get = swfdec_net_stream_get_variable;
-  asobject_class->mark = swfdec_net_stream_mark;
 }
 
 static void
@@ -541,24 +593,6 @@ swfdec_net_stream_init (SwfdecNetStream *stream)
   stream->buffer_time = 100; /* msecs */
 }
 
-SwfdecNetStream *
-swfdec_net_stream_new (SwfdecNetConnection *conn)
-{
-  SwfdecAsContext *context;
-  SwfdecNetStream *stream;
-  
-  g_return_val_if_fail (SWFDEC_IS_NET_CONNECTION (conn), NULL);
-
-  context = SWFDEC_AS_OBJECT (conn)->context;
-  if (!swfdec_as_context_use_mem (context, sizeof (SwfdecNetStream)))
-    return NULL;
-  stream = g_object_new (SWFDEC_TYPE_NET_STREAM, NULL);
-  swfdec_as_object_add (SWFDEC_AS_OBJECT (stream), context, sizeof (SwfdecNetStream));
-  stream->conn = conn;
-
-  return stream;
-}
-
 static void
 swfdec_net_stream_load (SwfdecPlayer *player, gboolean allowed, gpointer streamp)
 {
@@ -578,12 +612,14 @@ swfdec_net_stream_load (SwfdecPlayer *player, gboolean allowed, gpointer streamp
   stream->requested_url = NULL;
 }
 
+// When checkPolicyFile is true the YES values in last column must be changed
+// to POLICY
 static const SwfdecAccessMatrix swfdec_net_stream_matrix = {
   { SWFDEC_ACCESS_NO,  SWFDEC_ACCESS_NO,  SWFDEC_ACCESS_NO },
-  { SWFDEC_ACCESS_NO,  SWFDEC_ACCESS_YES, SWFDEC_ACCESS_POLICY },
+  { SWFDEC_ACCESS_NO,  SWFDEC_ACCESS_YES, SWFDEC_ACCESS_YES },
   { SWFDEC_ACCESS_YES, SWFDEC_ACCESS_NO,  SWFDEC_ACCESS_NO },
-  { SWFDEC_ACCESS_NO,  SWFDEC_ACCESS_NO,  SWFDEC_ACCESS_POLICY },
-  { SWFDEC_ACCESS_YES, SWFDEC_ACCESS_NO,  SWFDEC_ACCESS_POLICY }
+  { SWFDEC_ACCESS_NO,  SWFDEC_ACCESS_NO,  SWFDEC_ACCESS_YES },
+  { SWFDEC_ACCESS_YES, SWFDEC_ACCESS_NO,  SWFDEC_ACCESS_YES }
 };
 
 void
@@ -595,7 +631,7 @@ swfdec_net_stream_set_url (SwfdecNetStream *stream, const char *url_string)
   g_return_if_fail (SWFDEC_IS_NET_STREAM (stream));
   g_return_if_fail (url_string != NULL);
 
-  cx = SWFDEC_AS_OBJECT (stream)->context;
+  cx = swfdec_gc_object_get_context (stream);
   player = SWFDEC_PLAYER (cx);
 
   if (stream->requested_url != NULL) {
@@ -619,13 +655,13 @@ swfdec_net_stream_set_loader (SwfdecNetStream *stream, SwfdecLoader *loader)
 
   if (stream->loader) {
     SwfdecStream *lstream = SWFDEC_STREAM (stream->loader);
-    swfdec_stream_close (lstream);
+    swfdec_stream_ensure_closed (lstream);
     swfdec_stream_set_target (lstream, NULL);
     g_object_unref (lstream);
   }
   if (stream->flvdecoder) {
     g_signal_handlers_disconnect_by_func (stream->flvdecoder,
-	  swfdec_player_add_missing_plugin, SWFDEC_AS_OBJECT (stream)->context);
+	  swfdec_player_add_missing_plugin, swfdec_gc_object_get_context (stream));
     g_object_unref (stream->flvdecoder);
     stream->flvdecoder = NULL;
   }
@@ -693,10 +729,6 @@ swfdec_net_stream_seek (SwfdecNetStream *stream, double secs)
     SWFDEC_ERROR ("FIXME: implement seeking in audio only NetStream");
     return;
   }
-  if (stream->decoder) {
-    g_object_unref (stream->decoder);
-    stream->decoder = NULL;
-  }
   msecs = secs * 1000;
   msecs += first;
   if (msecs > last)
@@ -712,7 +744,7 @@ swfdec_net_stream_seek (SwfdecNetStream *stream, double secs)
     SWFDEC_WARNING ("FIXME: restarting audio after seek");
     swfdec_audio_remove (stream->audio);
     g_object_unref (stream->audio);
-    stream->audio = swfdec_audio_flv_new (SWFDEC_PLAYER (SWFDEC_AS_OBJECT (stream)->context), 
+    stream->audio = swfdec_audio_flv_new (SWFDEC_PLAYER (swfdec_gc_object_get_context (stream)), 
 	stream->flvdecoder, stream->current_time);
   }
 }
