@@ -107,7 +107,6 @@ struct _SwfdecPlayerPrivate
   GHashTable *		registered_classes;	/* name => SwfdecAsObject constructor */
 
   /* rendering */
-  SwfdecRectangle     	invalid_extents;      	/* extents of area that needs a redraw in global coordinates */
   GArray *		invalidations;		/* fine-grained areas in need of redraw */
   GSList *		invalid_pending;	/* pending invalidations due to invalidate_last */
   gboolean		fullscreen;		/* TRUE if the player has gone fullscreen */
@@ -248,7 +247,8 @@ void		swfdec_player_stop_sounds	(SwfdecPlayer *		player,
 						 gpointer		data);
 void		swfdec_player_stop_all_sounds	(SwfdecPlayer *		player);
 gboolean	swfdec_player_get_level		(SwfdecPlayer *		player,
-						 const char *		name);
+						 const char *		name,
+						 guint			version);
 SwfdecSpriteMovie *
 		swfdec_player_get_movie_at_level(SwfdecPlayer *		player,
 						 int			level);
@@ -264,8 +264,8 @@ void		swfdec_player_launch_with_headers (SwfdecPlayer *	player,
 						 const char *		target,
 						 SwfdecBuffer *		data,
 						 guint			header_count,
-						 const char **		header_names,
-						 const char **		header_values);
+						 char **		header_names,
+						 char **		header_values);
 void		swfdec_player_stage_to_global	(SwfdecPlayer *		player,
 						 double *		x,
 						 double *		y);
